#include "stdafx.h"
#include "SWPluginServerStruct.h"
#include "SWMessages.h"
#include "swhandle.h"
#include "ISWServerSettings.h"
#include "ISWServerData.h"
#include "swplugindllex.h"
#include <stdio.h>
#include <shellapi.h> // For ShellExecute
#include <stdlib.h>
#include <sstream>
#include "ISWPluginMenu.h"
#include "CustomData.h"
#include "TotalPingDlg.h"

//Don't remove the Object map macros. They make the ATL wizard work
BEGIN_OBJECT_MAP(ObjectMap)
END_OBJECT_MAP()
CAppModule _Module;
// end of ATL enabling defs

#define PLUGINEX_TABLE L"PLUGINEXTABLE"
#define DEFAULT_PROP_TABLE _T("PropertiesTable")

TotalPingDlg* pTotalPingDlg;

BOOL WINAPI DllMain( HANDLE hModule, 
                        DWORD fdwreason,  LPVOID lpReserved )
{
    g_hDll = (HINSTANCE)hModule; // Saved for later use
    switch(fdwreason) 
    {
        case DLL_PROCESS_ATTACH:
            // The DLL is being mapped into process's address space
            //  Do any required initialization on a per application basis,
            //  return FALSE if failed
            OnProcessAttached(g_hDll);
            break;
        case DLL_THREAD_ATTACH:
            break;
        case DLL_THREAD_DETACH:
            break;
        case DLL_PROCESS_DETACH:
            // The DLL unmapped from process's address space. Do necessary 
            // cleanup
            OnProcessDetached();
            break;
    }
    return TRUE;
}
// ----------------------------------------------------------------------------

bool DoYouSupport(const FunctionalSwitch eType)
{
    ATLTRACE(_T("DoYouSupport\n"));

    switch( eType )
    {
        case MENUS:
        case PING:
            return true;
        default:
            return false;
    }
}

bool DoYouExtend(const SWHandle* pluginHandle)
{
    return true;
}

SWRESULT Ping(SWPluginServerStruct* pThisServer)
{   
    // We know that the MyData structure won't be accessed accept here
    // so we didn't develop any thread safety mechanisms for this demo.
    // However, you most likely will need to.
    CustomData* pMyData = (CustomData*)pThisServer->pCustomData;
    pMyData->IncrementPings();

    // Protect the data integrity of the settings class
    pThisServer->pSettings->BeginRead();
    pThisServer->pServerData->BeginWrite();

    pThisServer->pServerData->AddRow( PLUGINEX_TABLE );
    pThisServer->pServerData->SetData( PLUGINEX_TABLE, 0, 
                                       pThisServer->pServerData->GetNumRowsUncommitted(PLUGINEX_TABLE) -1,
                                       pThisServer->pServerData->GetStatusText() );

    // Release the read lock
    pThisServer->pServerData->EndWrite();
    pThisServer->pSettings->EndRead();

    return SW_OK;
}

SWRESULT InitServer(SWPluginServerStruct* pThisServer)
{
    SWRESULT bRet = SW_OK;
	CString strHeaders;
    
    ISWServerData *pData = pThisServer->pServerData;

    // now set server specific settings (get them if they already existed, and
    // set defaults if they didn't exist.

    CustomData* pMyDataStruct = new CustomData;
    pThisServer->pCustomData  = (void*)pMyDataStruct;

    pData->BeginWrite();

    assert( pTotalPingDlg );
    pData->InsertPane( dynamic_cast<ISWPluginPane*>(pTotalPingDlg) );
    
    // Only setup the "Name" column. The network code
	// will have to handle the rest.
    pData->CreateTable( PLUGINEX_TABLE );
    pData->SetTableName( PLUGINEX_TABLE, L"PluginEx Table" );
    pData->SetUsersTableID( PLUGINEX_TABLE );

    pData->SetNumCols( PLUGINEX_TABLE, 1 );

    pData->SetColHeaderTitle( PLUGINEX_TABLE, 0, L"History" );
    pData->SetColDefaultWidth( PLUGINEX_TABLE, 0, 240 );

    pData->EndWrite();

    return bRet;
}

SWRESULT CleanupServer(SWPluginServerStruct* pThisServer)
{
    if (pThisServer->pCustomData != NULL)
    {
        delete (CustomData*)pThisServer->pCustomData;
        pThisServer->pCustomData = NULL;
    }

    return SW_OK;
}

SWRESULT LoadPlugin()
{
    ATLTRACE(_T("LoadPlugin\n"));

    pTotalPingDlg = new TotalPingDlg;

    if (pTotalPingDlg == NULL)
        return SW_MEMORYERR;

    pTotalPingDlg->Create( NULL );

    _ASSERT( ::IsWindow(pTotalPingDlg->GetPaneHWND()) );

    return SW_OK;
}

SWRESULT UnloadPlugin()
{
    ATLTRACE(_T("UnloadPlugin\n"));

    if( NULL != pTotalPingDlg )
    {
        if( ::IsWindow(pTotalPingDlg->GetPaneHWND()) )
        {
            ::DestroyWindow( pTotalPingDlg->GetPaneHWND() );
        }
            
        delete pTotalPingDlg;
        pTotalPingDlg = NULL;
    }

    return SW_OK;
}

void OnPluginExAction(SWPluginServerStruct* pThisServer,
           ISWPluginMenu::SWMenuType menuType, 
           ISWPluginMenu::ISWMenuContext* pMenuContext)
{
    if( pThisServer->pServerData->TableExists( DEFAULT_PROP_TABLE ) )
    {
        pThisServer->pServerData->BeginWrite();
        pThisServer->pServerData->AddRow( DEFAULT_PROP_TABLE );
        int nTotalRules = pThisServer->pServerData->GetNumRowsUncommitted( DEFAULT_PROP_TABLE );
        pThisServer->pServerData->SetData(DEFAULT_PROP_TABLE, 0, nTotalRules - 1, _T("PluginExAction"));
        pThisServer->pServerData->SetData(DEFAULT_PROP_TABLE, 1, nTotalRules - 1, _T("PluginExecutedAction") );
        pThisServer->pServerData->EndWrite();
    }
}

SWRESULT CreateMenu(SWPluginServerStruct* pThisServer, 
                        ISWPluginMenu* pPluginMenu)
{
    switch( pPluginMenu->GetMenuType() )
    {
        case ISWPluginMenu::UserList:
        case ISWPluginMenu::AuxStat:
            bool fDisabled = false;
            pPluginMenu->AddMenuItem( L"PluginExAction", OnPluginExAction, fDisabled?ISWPluginMenu_IsDisabled:0 );
            return SW_OK;
    }
    return SW_FAIL;
}