// NetworkDlg.cpp : Implementation of CNetworkDlg
#include "stdafx.h"
#include "SWMessages.h"
#include "swplugindll.h"
#include "CustomDataStruct.h"
#include "SWPluginServerStruct.h"
#include "NetworkDlg.h"
#include "DFNetUtil.h"
#include <stdio.h>
#include <limits.h>
#include <mswsock.h>
#include "SettingsDlg.h"

extern CSettingsDlg *g_pSettingsDlg;

/////////////////////////////////////////////////////////////////////////////
// CNetworkDlg

// constructor/destructor
CNetworkDlg::CNetworkDlg()
{
    m_pThisServer = NULL;
    m_sSocket = NULL;
	m_bCheckOK = true;
	m_uiRowNumLastEdited = 0;
	m_uiNumMessages = 0;
}

CNetworkDlg::~CNetworkDlg()
{
	WSACleanup();
    if(IsWindow())
	    DestroyWindow();
}

// Message Functions
LRESULT CNetworkDlg::OnInitDialog(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
{
    m_pThisServer = (SWPluginServerStruct*)lParam;
    ATLASSERT(m_pThisServer);

    // start up winsock
    // winsock initialization
    WORD version= MAKEWORD(2,2);
    WSADATA wsa_data;
    if ( WSAStartup(version,&wsa_data) != 0)
	{	
        //SetStatusText( _T("Couldn't initialize winsock.") );
        goto OnError;
    }

	m_rghSynchronize = new HANDLE[EVENT_ARRAY_SIZE];
    m_rghSynchronize[0] = CreateSemaphore(NULL, 0, LONG_MAX, NULL ); // Network read event
	m_rghSynchronize[1] = CreateSemaphore(NULL, 0, LONG_MAX, NULL ); // Network disconnect event
        
OnError:
	return 1;  // Let the system set the focus
}

LRESULT CNetworkDlg::OnDestroyDialog(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
{
	if( m_rghSynchronize )
	{
		CloseHandle( m_rghSynchronize[0] );
		CloseHandle( m_rghSynchronize[1] );
	}
	delete m_rghSynchronize;

	return 1; //not handling, just doing a little cleaning
}

LRESULT CNetworkDlg::OnAsynchronousMessage(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
{
    // Handle async messages here.
    int nAsyncEvent = WSAGETSELECTEVENT(lParam);
    int nAsyncError = WSAGETSELECTERROR(lParam);

	CustomDataStruct* pCustom = (CustomDataStruct*)m_pThisServer->pCustomData;
	if( CustomDataStruct::UNCONNECTED == pCustom->NetworkSequence )
	{
		//timeout went off first
		return 0;
	}

    if (nAsyncError)
    {
        CString strError;
		DFNetUtil::getStringFromWinsockError( WSAGetLastError(), strError );	
		SetError( strError );
		CleanupNetwork();
        return 0;
    }
    switch(nAsyncEvent)
    {
        case FD_READ:
            ReleaseSemaphore(m_rghSynchronize[0], 1, NULL);
            break;
		case FD_CLOSE:
			ReleaseSemaphore(m_rghSynchronize[1], 1, NULL);
            break;
    };

    return 0;
}

void CNetworkDlg::CleanupNetwork()
{
    int nRet = WSAAsyncSelect( m_sSocket, this->m_hWnd, 0, 0 );
    if (nRet == SOCKET_ERROR)
    {
        CString strError;
        DFNetUtil::getStringFromWinsockError( WSAGetLastError(), strError );
    }

    closesocket(m_sSocket);
    m_sSocket = NULL;

	((CustomDataStruct*)m_pThisServer->pCustomData)->NetworkSequence = CustomDataStruct::UNCONNECTED;

    ::SendMessage(g_pSettingsDlg->GetPaneHWND(), 
                  SWPN_DATAREADY, (WPARAM)&(m_pThisServer->swhHandle), SW_UPDATE_DATA);
}

int CNetworkDlg::DealWithPacket(char *rawpkt, int pktlen)
{
    int pktct = 0;
	CString strRconReply;
	bool bWasSuccess;
	bool bForceSendQuit = false;
	CustomDataStruct* pCustom = (CustomDataStruct*)m_pThisServer->pCustomData;

	if( m_bCheckOK )
	{
		if( '+' == *rawpkt )
		{
			bWasSuccess = true;
			pktct = 4;
		}
		else
		{
			bWasSuccess = false;
			'-' == *rawpkt ? pktct = 5 : pktct = 0;
			if( 5 == pktct )
			{
				int ct=pktct;
				for( ; rawpkt[ct] != '\r'; ct++ );
				rawpkt[ct] = '\0';
			}
			else
			{
				//we don't know what is in this, probably gibberish
				rawpkt[50] = '\0';
			}
			SetError( &rawpkt[pktct] );
			bForceSendQuit = true;
		}
	}

	if( !bForceSendQuit )
	{
		// handle/send packets
		SWRESULT tester;
		switch (pCustom->NetworkSequence)
		{
			case CustomDataStruct::CONNECT:
				tester = OnConnect( rawpkt, pktct );
				break;
			case CustomDataStruct::USERNAME:
				tester = OnUserName( rawpkt, pktct );
				break;
			case CustomDataStruct::PASSWORD:
				tester = OnPassword( rawpkt, pktct );
				break;
			case CustomDataStruct::STATUS:
				tester = OnStatus( rawpkt, pktct );
				break;
			case CustomDataStruct::LIST:
				tester = OnList( rawpkt, pktct );
				break;
			case CustomDataStruct::TOP:
				tester = OnTop( rawpkt, pktct );
				break;
			case CustomDataStruct::DISCONNECT:
				tester = OnDisconnect( rawpkt, pktct );
				break;
			default:
				m_strSendBuffer.Empty();
				SetError(_T("Unknown Error Occured"));
				bForceSendQuit = true;
				break;
		}

		if( tester != SW_OK )
		{
			bForceSendQuit = true;
		}
	}

	if( true == bForceSendQuit )
	{
		pCustom->NetworkSequence = CustomDataStruct::DISCONNECT;
		m_strSendBuffer = "QUIT\r\n";
	}


	if( !m_strSendBuffer.IsEmpty() )
	{
		// TOP may have poorly formed responses, prehandle it
		if( pCustom->NetworkSequence == CustomDataStruct::TOP )
		{
			//fill buffer with '.' so we can tell if TOP
			//has been sent as multiple packets
			for( int ct=0; ct < BUFFER_LEN-1; ct++ )
			{
				rawpkt[ct] = '.';
			}
		}

		int nRet;
		nRet = send(m_sSocket, m_strSendBuffer.GetBuffer(3), m_strSendBuffer.GetLength(), NULL );
		if( nRet == SOCKET_ERROR )
		{
			CString strError;
			DFNetUtil::getStringFromWinsockError( WSAGetLastError(), strError );	
			SetError( strError );
			CleanupNetwork();
			return SW_FAIL;
		}
	}

	return SW_OK;
}

SWRESULT CNetworkDlg::Ping()
{
    CustomDataStruct* pCustom = (CustomDataStruct*)m_pThisServer->pCustomData;
    ISWServerSettings* pSettings = m_pThisServer->pSettings;
    CString strAddress;
	unsigned long server_ipaddr;
    struct sockaddr_in sockaddress;
    int nPort;
    int nRet;
	int nTimeout;
    DWORD dwResult;
	bool cont = true;

	// Make sure the semaphores are cleared out
	while( WAIT_TIMEOUT != WaitForSingleObject( m_rghSynchronize[0], 0 ) );
	while( WAIT_TIMEOUT != WaitForSingleObject( m_rghSynchronize[1], 0 ) );
    
    m_sSocket = socket( AF_INET, SOCK_STREAM, IPPROTO_TCP);
    if (m_sSocket == INVALID_SOCKET) 
    {
        goto rcon_send_failed;
    }

    // request event notification, set socket non-blocking.
    nRet = WSAAsyncSelect( m_sSocket, this->m_hWnd, IDM_ASYNC, FD_READ|FD_CLOSE );
    if (nRet == SOCKET_ERROR)
    {
        goto rcon_send_failed;
    }

    // open socket for sending
    pSettings->BeginRead();
    strAddress = pSettings->GetServerAddress().c_str();
    nPort = pSettings->GetPort();
	nTimeout = pSettings->GetTimeout() * 1000;
    pSettings->EndRead();

	pCustom->NetworkSequence = CustomDataStruct::CONNECT;
	m_bCheckOK = true;
    
	//Get Server Info
	if( SW_OK != DFNetUtil::getServerFromString(strAddress, server_ipaddr) )
    {
		goto rcon_send_failed;
    }

    sockaddress.sin_addr.s_addr = server_ipaddr;
    sockaddress.sin_family = AF_INET;
    sockaddress.sin_port = htons(nPort);
    memset( &(sockaddress.sin_zero), 0, sizeof(sockaddress.sin_zero) );
    nRet = connect( m_sSocket, (LPSOCKADDR)&sockaddress, SOCKADDR_LEN);
    if( WSAEWOULDBLOCK != WSAGetLastError() )
    {
		goto rcon_send_failed;
    }
    
    // The message loop lasts until we get a WM_QUIT message,
    // upon which we shall return from the function.
	pCustom->pServerData->BeginWrite();
	
	// wait for the connect
	dwResult = MsgWaitForMultipleObjects( EVENT_ARRAY_SIZE, m_rghSynchronize, FALSE,
                                              nTimeout, QS_ALLINPUT ); 

	if( dwResult == WAIT_TIMEOUT )
	{
		if( WAIT_TIMEOUT == dwResult )
		{
			SetError("Couldn't connect to server.");
		}
	}
	else
	{
		do
		{
			// The result tells us the type of event we have.
			if( dwResult == WAIT_OBJECT_0 )
			{
				OnNetworkRead();
				if( CustomDataStruct::UNCONNECTED == pCustom->NetworkSequence )
				{ 
					cont = false;
				}

			}
			else if( dwResult == ( WAIT_OBJECT_0 + 1 ) )
			{
				// Servers like to disconnect when there are errors
				// If the sequesnce is set to DISCONNECT we are already
				// in the disconnect phase (likely because of an error
				// with the server) so we don't want to write a new error
				// so that the real error from the server is shown.
				if( CustomDataStruct::DISCONNECT != pCustom->NetworkSequence )
				{
					SetError("Lost connection with server.");
				}
				cont = false;
			}
			
			// Wait for any message sent or posted to this queue 
			// or for one of the passed handles be set to signaled.
			if( cont )
			{
				dwResult = MsgWaitForMultipleObjects( EVENT_ARRAY_SIZE, m_rghSynchronize, FALSE,
													INFINITE, QS_ALLINPUT ); 
			}
		}while (cont);
	}

	pCustom->pServerData->EndWrite();

	CleanupNetwork();

    return SW_OK;

// Function ends this way if there is an error
rcon_send_failed:
	CString strError;
	DFNetUtil::getStringFromWinsockError( WSAGetLastError(), strError );	
	SetError( strError );
	CleanupNetwork();
	return SW_FAIL;
}

void CNetworkDlg::SetError( const CString &strErrMessage )
{
	CustomDataStruct* pCustom = (CustomDataStruct*)m_pThisServer->pCustomData;

	if( pCustom->NetworkSequence != CustomDataStruct::UNCONNECTED || 
		pCustom->NetworkSequence != CustomDataStruct::DISCONNECT )
	{
		pCustom->pServerData->BeginWrite();
    
		// Clear the table
        pCustom->pServerData->ResetData(DEFAULT_TABLE);

		// Clear aux stat (size in octets)
		pCustom->pServerData->SetAuxStat( L"" );

		// Set status	
        pCustom->pServerData->SetStatusText( strErrMessage.GetString() );

		// Set icon to down
		pCustom->pServerData->SetIconDown();

		pCustom->pServerData->EndWrite();
	}
}

// If this function returns SW_FAIL, handle the network error
// yourself including any calls to WSAGetLastError()
SWRESULT CNetworkDlg::NextRecv( char* rawpkt, int iFirstUnprocessedCell, const int iBufferLen )
{
	int iTester;
	int i = iFirstUnprocessedCell;
	for( iTester=0; i < iBufferLen; i++, iTester++ )
	{
		rawpkt[iTester] = rawpkt[i];
	}

	DWORD dwResult;
	dwResult = MsgWaitForMultipleObjects( EVENT_ARRAY_SIZE, m_rghSynchronize, FALSE, INFINITE, QS_ALLINPUT );
	if( dwResult == ( WAIT_OBJECT_0 + 1 ) )
    {
        SetError("Lost connection with server.");
		return SW_FAIL;
    }

	int flag = 0;
	int nRet = WSARecvEx(m_sSocket, &rawpkt[iTester], iBufferLen-iTester, &flag);
	if( nRet == SOCKET_ERROR)
	{
		return SW_FAIL;
	}

	return SW_OK;
}

SWRESULT CNetworkDlg::OnConnect( char* rawpkt, int pktct )
{
    CustomDataStruct* pCustom = (CustomDataStruct*)m_pThisServer->pCustomData;

	WaitForSingleObject( pCustom->mutex, INFINITE );
	m_uiNumMessages = 0;
	pCustom->NetworkSequence = CustomDataStruct::USERNAME;
	m_strSendBuffer = "USER ";
	m_strSendBuffer += pCustom->strUsername;
	m_strSendBuffer += "\r\n";
	ReleaseMutex( pCustom->mutex );

	return SW_OK;
}

SWRESULT CNetworkDlg::OnUserName( char* rawpkt, int pktct )
{
    CustomDataStruct* pCustom = (CustomDataStruct*)m_pThisServer->pCustomData;

	WaitForSingleObject( pCustom->mutex, INFINITE );
	pCustom->NetworkSequence = CustomDataStruct::PASSWORD;
	m_strSendBuffer = "PASS ";
	m_strSendBuffer += pCustom->strPassword;
	m_strSendBuffer += "\r\n";
	ReleaseMutex( pCustom->mutex );

	return SW_OK;
}

SWRESULT CNetworkDlg::OnPassword( char* rawpkt, int pktct )
{
    CustomDataStruct* pCustom = (CustomDataStruct*)m_pThisServer->pCustomData;

	pCustom->NetworkSequence = CustomDataStruct::STATUS;
	m_strSendBuffer = "STAT\r\n";

	return SW_OK;
}

SWRESULT CNetworkDlg::OnStatus( char* rawpkt, int pktct )
{
    CustomDataStruct* pCustom = (CustomDataStruct*)m_pThisServer->pCustomData;

	int ct=0;
	for( ct=pktct; rawpkt[ct] != ' '; ct++ );
	rawpkt[ct] = '\0';
	m_uiNumMessages = atoi(&rawpkt[pktct]);
	pCustom->pServerData->SetIconNumber( m_uiNumMessages );
	int oldCt = ct;
	for( ct++; rawpkt[ct] != '\r' && rawpkt[ct] != ' '; ct++ );
	rawpkt[ct] = '\0';
    pCustom->pServerData->SetAuxStat( CString(&rawpkt[oldCt+1]).GetString() );

	if( 0 != m_uiNumMessages )
	{
		pCustom->NetworkSequence = CustomDataStruct::LIST;
		m_strSendBuffer = "LIST\r\n";
	}
	else
	{
        pCustom->pServerData->ResetData(DEFAULT_TABLE);

		pCustom->NetworkSequence = CustomDataStruct::DISCONNECT;
		m_strSendBuffer = "QUIT\r\n";
	}

	return SW_OK;
}

SWRESULT CNetworkDlg::OnList( char* rawpkt, int pktct )
{
    CustomDataStruct* pCustom = (CustomDataStruct*)m_pThisServer->pCustomData;

	int ct=0;
	int oldCt=ct;
	m_uiRowNumLastEdited = 0;
	pCustom->pServerData->ResetData(DEFAULT_TABLE);
	for( ct=0; rawpkt[ct] != '\n'; ct++ );
	ct++;

	bool bContinue = true;
	while( bContinue )
	{
		// Make sure the full line is in the buffer
		int i;
		for( i = ct; i < BUFFER_LEN && rawpkt[i] != '\n'; i++ );
		if( BUFFER_LEN == i )
		{
			if( SW_FAIL == NextRecv(rawpkt,ct,BUFFER_LEN) )
			{
				CString strError;
				DFNetUtil::getStringFromWinsockError( WSAGetLastError(), strError );
				SetError( strError );
				return SW_FAIL;
			}
			// reset ct to front of packet
			ct = 0;
		}

		//A line should be at least 3 character - [some char][\r][\n]
		if( rawpkt[ct] == '.' && rawpkt[ct+1] == '\r' )
		{ 
			bContinue = false;
		}
		else
		{
            pCustom->pServerData->AddRow(DEFAULT_TABLE);
			for( ; rawpkt[ct] != ' '; ct++ );
			oldCt = ++ct;
			for( ; rawpkt[ct] != '\r' && rawpkt[ct] != ' '; ct++ );
			rawpkt[ct] = '\0';
            pCustom->pServerData->SetData( DEFAULT_TABLE, 1, m_uiRowNumLastEdited, CString(&rawpkt[oldCt]).GetString() );
			m_uiRowNumLastEdited++;
			for( ; rawpkt[ct] != '\n'; ct++ );
			ct++;
		}
	}
	m_uiRowNumLastEdited = 0;
	
	pCustom->NetworkSequence = CustomDataStruct::TOP;
	m_strSendBuffer = "TOP 1 0\r\n";

	return SW_OK;
}

SWRESULT CNetworkDlg::OnTop( char* rawpkt, int pktct )
{
    CustomDataStruct* pCustom = (CustomDataStruct*)m_pThisServer->pCustomData;

	if( '+' == *rawpkt )
	{
		int iTester;
		for( iTester=0; rawpkt[iTester] != '\n' && iTester < BUFFER_LEN; iTester++ );
		if( '.' == rawpkt[iTester+1] )
		{
			m_bCheckOK = false;
			m_strSendBuffer.Empty();
			return SW_OK;
		}
	}
	else if( '-' == *rawpkt )
	{
		pktct = 5;
		int ct=pktct;
		for( ; rawpkt[ct] != '\r'; ct++ );
		rawpkt[ct] = '\0';
		SetError( &rawpkt[pktct] );
		return SW_FAIL;
	}

	int ct=0;
	int oldCt=ct;
	bool bContinue = true;
	while( bContinue )
	{
		//test whether this line has been fully recieved.
		//if it hasn't, store what has been recieved,
		//get the rest
		int i;
		for( i = ct; i < BUFFER_LEN && rawpkt[i] != '\n'; i++ );
		if( BUFFER_LEN == i )
		{
			if( SW_FAIL == NextRecv(rawpkt,ct,BUFFER_LEN) )
			{
				CString strError;
				DFNetUtil::getStringFromWinsockError( WSAGetLastError(), strError );
				SetError( strError );
				return SW_FAIL;
			}
			// reset ct to front of packet
			ct = 0;
		}

		//A line should be at least 3 character - [some char][\r][\n]
		if( rawpkt[ct] == '.' )
		{ 
			if( rawpkt[ct+1] == '\r' )
			{
				bContinue = false;
			}
			else
			{
				ct++;
			}
		}
		else
		{
			//if from
			if( 'F' == toupper(rawpkt[ct]) )
			{
				rawpkt[ct+5] = '\0';
				if( 0 == stricmp("From:",&rawpkt[ct]) )
				{
					bool bForeignCharSet = false;
					bool bIgnoreCurrentChar = false;
					char strFrom[SCENTENCE_LEN] = {0};
					int i=ct+6;
					for( int i2=0; i2<SCENTENCE_LEN; i++ )
					{
						if( '\r' == rawpkt[i] )
						{
							strFrom[i2] = '\0';
							break;
						}
						else if( '<' == rawpkt[i] )
						{
							// Block the email address, but only if there
							// was a name before it
							if( 0 < i2 )
							{
								strFrom[i2-1] = '\0';
								break;
							}
							else if( bForeignCharSet )
							{
								bIgnoreCurrentChar = false;
							}
						}
						else if( '>' == rawpkt[i] )
						{
							// this should only come into play if there
							// was no name and wil only have "<address>"
							strFrom[i2] = '\0';
							break;
						}
						else if( '=' == rawpkt[i] && 0 == i2 )
						{
							if( 0 == strnicmp("=?iso-8859-1?", &rawpkt[i], 13) )
							{
								bForeignCharSet = true;
								bIgnoreCurrentChar = true;
							}
						}
						else if( '"' != rawpkt[i] && !bIgnoreCurrentChar )
						{
							strFrom[i2] = rawpkt[i];
							i2++;
						}
					}
					ct = i;
					strFrom[SCENTENCE_LEN-1] = '\0';
                    pCustom->pServerData->SetData( DEFAULT_TABLE, 0, m_uiRowNumLastEdited, CString(strFrom).GetString() );
				}
			}
			//if date
			else if( 'D' == toupper(rawpkt[ct]) )
			{
				rawpkt[ct+5] = '\0';
				if( 0 == stricmp("Date:",&rawpkt[ct]) )
				{
					oldCt = ct += 6;
					for( ; '\r' != rawpkt[ct]; ct++ );
					rawpkt[ct] = '\0';
					pCustom->pServerData->SetData( DEFAULT_TABLE, 3, m_uiRowNumLastEdited, CString(&rawpkt[oldCt]).GetString() );
				}
			}
			//if subject
			else if( 'S' == toupper(rawpkt[ct]) )
			{
				rawpkt[ct+8] = '\0';
				if( 0 == stricmp("Subject:",&rawpkt[ct]) )
				{
					oldCt = ct += 9;
					for( ; '\r' != rawpkt[ct]; ct++ );
					rawpkt[ct] = '\0';
					pCustom->pServerData->SetData( DEFAULT_TABLE, 2, m_uiRowNumLastEdited, CString(&rawpkt[oldCt]).GetString() );
				}
			}
			for( ; '\n' != rawpkt[ct]; ct++ );
			ct++;
		}
	}
		
	m_uiRowNumLastEdited++;

	m_bCheckOK = true;
	if( m_uiRowNumLastEdited < m_uiNumMessages )
	{
		// Don't need to set TOP as it is already TOP
		m_strSendBuffer.Format("TOP %d 0\r\n", m_uiRowNumLastEdited+1);
	}
	else
	{
		pCustom->NetworkSequence = CustomDataStruct::DISCONNECT;
		m_strSendBuffer = "QUIT\r\n";
	}

	return SW_OK;
}

SWRESULT CNetworkDlg::OnDisconnect( char* rawpkt, int pktct )
{
    CustomDataStruct* pCustom = (CustomDataStruct*)m_pThisServer->pCustomData;

	CString strStatus;
	strStatus.Format( _T("%d messages"), m_uiNumMessages );
	pCustom->pServerData->SetStatusText( strStatus.GetString() );
	pCustom->pSettings->BeginRead();
	strStatus += _T(" on ");
	strStatus += pCustom->pSettings->GetUserSetServerName().c_str();
	pCustom->pSettings->EndRead();
	pCustom->pServerData->SetIconTip( strStatus.GetString() );
	m_strSendBuffer.Empty();
	pCustom->NetworkSequence = CustomDataStruct::UNCONNECTED;

	return SW_OK;
}

void CNetworkDlg::OnNetworkRead()
{
	int nRet, flags=0;
	CustomDataStruct* pCustomData = (CustomDataStruct*)m_pThisServer->pCustomData;
	char* pChallengeNumber = NULL;

	nRet = WSARecvEx(m_sSocket, szRecvBuffer, 
					 sizeof(szRecvBuffer), &flags);
	if( nRet == SOCKET_ERROR)
	{
		CString strError;
		DFNetUtil::getStringFromWinsockError( WSAGetLastError(), strError );
		SetError( strError );
		CleanupNetwork();
	}
	else
	{
		DealWithPacket( szRecvBuffer, nRet );
	}
}
