// SettingsDlg.cpp : Implementation of CSettingsDlg
#include "stdafx.h"
#include "SWMessages.h"
#include "DFRegTool.h"

#include "SWPluginServerStruct.h"
#include "CustomDataStruct.h"
#include "SettingsDlg.h"

/////////////////////////////////////////////////////////////////////////////
// CSettingsDlg

CSettingsDlg::CSettingsDlg()
{
    m_pCurrentServer = NULL;
    m_fSettingsChanged = FALSE;
    m_fRememberPassUser = FALSE;
    m_hwndParent = NULL;
    m_hwndPassword = NULL; 
    m_hwndRemember = NULL;
    m_hwndUsername = NULL;
    m_strPassword = "";
	m_strUsername = "";
}

CSettingsDlg::~CSettingsDlg()
{
}

LRESULT CSettingsDlg::OnInitDialog(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
{
    m_hwndPassword = GetDlgItem( IDC_PASSWORD );
	m_hwndRemember = GetDlgItem( IDC_REMEMBERPASSWORD );
    m_hwndUsername = GetDlgItem( IDC_USERNAME );

    m_fSettingsChanged = FALSE;
	return 1;  // Let the system set the focus
}

LRESULT CSettingsDlg::OnSetDialog(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
{
    m_hwndParent = GetParent().m_hWnd;
    _ASSERT(m_hwndParent);
    // get the settings for this particular settings from wParam, and then show
    // them according to the flags in lParam
    
    // TODO: Get the settings struct out of the hashtable
    
    m_pCurrentServer = (SWPluginServerStruct*)wParam;
    
    if (!m_pCurrentServer)
        return 0;

    if (lParam & SW_UPDATE_DATA)
    {
        // Update the Server Data here
    }

    if (lParam & SW_UPDATE_SETTINGS)
    {
        // Update the Settings here
        CustomDataStruct* pCustom = (CustomDataStruct*)m_pCurrentServer->pCustomData;

		WaitForSingleObject( pCustom->mutex, INFINITE );
        m_fRememberPassUser = pCustom->fRememberPassUser;
        m_strPassword = pCustom->strPassword;
		m_strUsername = pCustom->strUsername;
		ReleaseMutex( pCustom->mutex );

        // HACKHACK: we don't want to enable the apply buttons, so say we have already
        // set the apply buttons, and then after this setwindow text, set the settings
        // changed to false;
        m_fSettingsChanged = TRUE;
        ::SetWindowText(m_hwndPassword, m_strPassword);
        ::SetWindowText(m_hwndUsername, m_strUsername);
        if( m_fRememberPassUser )
            ::SendMessage(m_hwndRemember, BM_SETCHECK, BST_CHECKED, 0);
        else
            ::SendMessage(m_hwndRemember, BM_SETCHECK, BST_UNCHECKED, 0);
        m_fSettingsChanged = FALSE;
    }

    return 0;
}
    
LRESULT CSettingsDlg::OnChangesApplied(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
{
    // Applies the changes on the currently selected dialog.
    ATLTRACE("Changes applied in plugin");

    if (m_fSettingsChanged)
    {
        CustomDataStruct* pCustom = (CustomDataStruct*)m_pCurrentServer->pCustomData;
        pCustom->fRememberPassUser = m_fRememberPassUser ? true : false;
		WaitForSingleObject( pCustom->mutex, INFINITE );
        pCustom->strPassword = m_strPassword;
		pCustom->strUsername = m_strUsername;
		ReleaseMutex( pCustom->mutex );

		DFRegTool reg;
		reg.OpenKey(m_pCurrentServer->strRegistryBase);
        if( m_fRememberPassUser )
		{
            reg.AddCodedStr( _T("Password"), m_strPassword, 556, 134);
			reg.AddCodedStr( _T("Username"), m_strUsername, 495, 124);
		}
        else
		{
            reg.AddCodedStr( _T("Password"), _T(""), 556, 134);
            reg.AddCodedStr( _T("Username"), _T(""), 495, 124);
		}
		reg.AddDword( _T("Remember"), m_fRememberPassUser );
		reg.CloseKey();
		
        // reset the settings changed
        m_fSettingsChanged = FALSE;
        ::SendMessage(m_hwndParent, SWPN_ENABLE_APPLY, m_fSettingsChanged, 0);
        ::SendMessage(m_hwndParent, SWPN_ENABLE_RESET, m_fSettingsChanged, 0);
    }
    return 0;
}

LRESULT CSettingsDlg::OnChangesReset(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
{
    // Resets the changes on the currently selected dialog.
    ATLTRACE(_T("Changes reset in plugin"));
    _ASSERT(m_fSettingsChanged);

	SetData();
    
    ::SetWindowText( m_hwndPassword, m_strPassword );
    ::SetWindowText( m_hwndUsername, m_strUsername );
	if( m_fRememberPassUser )
        ::SendMessage(m_hwndRemember, BM_SETCHECK, BST_CHECKED, 0);
    else
        ::SendMessage(m_hwndRemember, BM_SETCHECK, BST_UNCHECKED, 0);

    // set the Apply and Reset buttons
    m_fSettingsChanged = FALSE;
    ::SendMessage(m_hwndParent, SWPN_ENABLE_APPLY, m_fSettingsChanged, 0);
    ::SendMessage(m_hwndParent, SWPN_ENABLE_RESET, m_fSettingsChanged, 0);

    return 0;
}
    
LRESULT CSettingsDlg::OnChangePassword(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled)
{
	int len = ::GetWindowTextLength(m_hwndPassword);
	::GetWindowText(m_hwndPassword, m_strPassword.GetBufferSetLength(len), len +1); 

	// The setting just changed, so tell server watch that we are in need of an update
    if (!m_fSettingsChanged)
    {
        m_fSettingsChanged = TRUE;
        ::SendMessage(m_hwndParent, SWPN_ENABLE_APPLY, m_fSettingsChanged, 0);
        ::SendMessage(m_hwndParent, SWPN_ENABLE_RESET, m_fSettingsChanged, 0);
    }
	return 0;
}

LRESULT CSettingsDlg::OnChangeUsername(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled)
{
	int len = ::GetWindowTextLength(m_hwndUsername);
	::GetWindowText(m_hwndUsername, m_strUsername.GetBufferSetLength(len), len +1); 

	// The setting just changed, so tell server watch that we are in need of an update
    if (!m_fSettingsChanged)
    {
        m_fSettingsChanged = TRUE;
        ::SendMessage(m_hwndParent, SWPN_ENABLE_APPLY, m_fSettingsChanged, 0);
        ::SendMessage(m_hwndParent, SWPN_ENABLE_RESET, m_fSettingsChanged, 0);
    }
	return 0;
}

LRESULT CSettingsDlg::OnClickedRememberpassword(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled)
{
	m_fRememberPassUser = !m_fRememberPassUser;

	// The setting just changed, so tell server watch that we are in need of an update
    if (!m_fSettingsChanged)
    {
        m_fSettingsChanged = TRUE;
        ::SendMessage(m_hwndParent, SWPN_ENABLE_APPLY, m_fSettingsChanged, 0);
        ::SendMessage(m_hwndParent, SWPN_ENABLE_RESET, m_fSettingsChanged, 0);
    }
	return 0;
}

LRESULT CSettingsDlg::OnOK(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled)
{
	DestroyWindow();
	return 0;
}

LRESULT CSettingsDlg::OnCancel(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled)
{
	DestroyWindow();
	return 0;
}

SWRESULT CSettingsDlg::SetData()
{
    long lpResult;
	DFRegTool reg;

    if( m_pCurrentServer )
    {
        reg.OpenKey(m_pCurrentServer->strRegistryBase);
	    lpResult = reg.GetCodedStr( _T("Password"), m_strPassword, 556, 134 );
	    if( lpResult != ERROR_SUCCESS )
		    m_strPassword = _T("");
	    
        lpResult = reg.GetCodedStr(_T("Username"), m_strUsername, 495, 124);
        if( lpResult != ERROR_SUCCESS )
            m_strUsername = _T("");

	    int iHelper;
	    lpResult = reg.GetDword( _T("Remember"), iHelper );
	    if( lpResult != ERROR_SUCCESS )
        {
            m_fRememberPassUser = FALSE;
        }
        else
	    {
		    m_fRememberPassUser = (BOOL)iHelper;
	    }
	    reg.CloseKey();

        CustomDataStruct* pCustom = (CustomDataStruct*)m_pCurrentServer->pCustomData;
		WaitForSingleObject( pCustom->mutex, INFINITE );
        pCustom->fRememberPassUser = m_fRememberPassUser ? true : false;
        pCustom->strPassword = m_strPassword;
        pCustom->strUsername = m_strUsername;
		ReleaseMutex( pCustom->mutex );
    }

	return SW_OK;
}

LRESULT CSettingsDlg::OnNotifyFormat(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
{
    return NFR_UNICODE;
}

const std::wstring& CSettingsDlg::GetTitle() const
{
    static std::wstring strTitle = L"Login Info";

    return strTitle;
}

ISWPluginPane::DisplayMode CSettingsDlg::GetDisplayMode() const
{
    return ISWPluginPane::Settings;
}

HWND CSettingsDlg::GetPaneHWND() const
{
    return m_hWnd;
}

bool CSettingsDlg::IsResizeable() const
{
    return true;
}

void CSettingsDlg::PrePaneSelection()
{
}