#include "stdafx.h"
#include "SWMessages.h"
#include "swhandle.h"
#include "ISWServerSettings.h"
#include "ISWServerData.h"
#include "SettingsDlg.h"
#include "swplugindll.h"
#include "SWPluginServerStruct.h"
#include <stdio.h>
#include <shellapi.h> // For ShellExecute
#include <stdlib.h>
#include "DFRegTool.h"
#include "CustomDataStruct.h"

//Don't remove the Object map macros. They make the ATL wizard work
BEGIN_OBJECT_MAP(ObjectMap)
END_OBJECT_MAP()
CAppModule _Module;
// end of ATL enabling defs

CSettingsDlg *g_pSettingsDlg = NULL;

BOOL WINAPI DllMain( HANDLE hModule, 
                        DWORD fdwreason,  LPVOID lpReserved )
{
    g_hDll = (HINSTANCE)hModule; // Saved for later use
    switch(fdwreason) 
    {
        case DLL_PROCESS_ATTACH:
            // The DLL is being mapped into process's address space
            //  Do any required initialization on a per application basis,
            //  return FALSE if failed
            OnProcessAttached(g_hDll);
            break;
        case DLL_THREAD_ATTACH:
            break;
        case DLL_THREAD_DETACH:
            break;
        case DLL_PROCESS_DETACH:
            // The DLL unmapped from process's address space. Do necessary 
            // cleanup
            OnProcessDetached();
            break;
    }
    return TRUE;
}
// ----------------------------------------------------------------------------

bool DoYouSupport(const FunctionalSwitch eType)
{
    switch( eType )
    {
        case AUX_STAT:
        case BUDDIES:
        case JOIN:
            return true;
        default:
            return false;
    }
}

SWRESULT Ping(SWPluginServerStruct* pThisServer)
{   
    ISWServerData* pData = pThisServer->pServerData;
    ISWServerSettings* pSettings = pThisServer->pSettings;
    CString strStatus;
    CString strAddress;
    
    pSettings->Retain();
    pData->Retain();

    CustomDataStruct* pCustom = (CustomDataStruct*)pThisServer->pCustomData;
    
    // Actually ping the server
    pData->BeginWrite();
    pCustom->pAdminNetwork->Ping();
    pData->EndWrite();

    pSettings->Release();
    pData->Release();
    return SW_OK;
}

SWRESULT InitServer(SWPluginServerStruct* pThisServer)
{
    long lpResult;
	SWRESULT bRet = SW_OK;
    ISWServerSettings* pSettings = pThisServer->pSettings; 
    ISWServerData* pServerData = pThisServer->pServerData;
    CString strHeaders;
    DFRegTool reg;

    // now set server specific settings (get them if they already existed, and
    // set defaults if they didn't exist.
    //pInsertStruct->nPingRetries = 4;  // number of default retries for the new server
    CustomDataStruct* pCustomData = new CustomDataStruct;
    pCustomData->pAdminNetwork = new CNetworkDlg;
    pCustomData->pAdminNetwork->Create( GetParentHWND(), (LPARAM)pThisServer);
	pCustomData->pSettings = pSettings;
	pCustomData->pServerData = pServerData;
	pCustomData->pSettings->Retain();
	pCustomData->pServerData->Retain();
    pThisServer->pCustomData = (void*)pCustomData;

    pServerData->BeginWrite();
    
    // Setup the panes
    if( NULL != g_pSettingsDlg && ::IsWindow(g_pSettingsDlg->GetPaneHWND()) )
    {
        pServerData->InsertPane( g_pSettingsDlg );
    }

    // Set up the players Table columns.
    pServerData->CreateTable( DEFAULT_TABLE );
    pServerData->SetTableName( DEFAULT_TABLE, L"Players" );
    pServerData->SetUsersTableID( DEFAULT_TABLE );
    pServerData->SetNumCols( DEFAULT_TABLE, 4 );
    pServerData->SetColHeaderTitle( DEFAULT_TABLE, 0, L"From" );
    pServerData->SetColDefaultWidth( DEFAULT_TABLE, 0, 150 );
    pServerData->SetColHeaderTitle( DEFAULT_TABLE, 1, L"Size" );
    pServerData->SetColDefaultWidth( DEFAULT_TABLE, 1, 60 );
    pServerData->SetColHeaderTitle( DEFAULT_TABLE, 2, L"Subject" );
    pServerData->SetColDefaultWidth( DEFAULT_TABLE, 2, 53 );
    pServerData->SetColHeaderTitle( DEFAULT_TABLE, 3, L"Date" );
    pServerData->SetColDefaultWidth( DEFAULT_TABLE, 3, 40 );

    pServerData->EndWrite();

    // initialize plugin specific settings.
	//  can't use the protection mutex here, don't need it either
    reg.OpenKey(pThisServer->strRegistryBase);
	lpResult = reg.GetCodedStr( _T("Password"), pCustomData->strPassword, 556, 134 );
	if( lpResult != ERROR_SUCCESS )
		pCustomData->strPassword.Empty();
	
    reg.OpenKey(pThisServer->strRegistryBase);
	lpResult = reg.GetCodedStr( _T("Username"), pCustomData->strUsername, 495, 124 );
	if( lpResult != ERROR_SUCCESS )
		pCustomData->strUsername.Empty();

	int iHelper;
	lpResult = reg.GetDword( _T("Remember"), iHelper );
	if( lpResult != ERROR_SUCCESS )
    {
        pCustomData->fRememberPassUser = FALSE;
    }
    else
	{
        pCustomData->fRememberPassUser = iHelper ? true : false;
	}
	reg.CloseKey();
        
    return bRet;
}

SWRESULT CleanupServer(SWPluginServerStruct* pThisServer)
{
    if (pThisServer)
    {
        CustomDataStruct* pCustom = (CustomDataStruct*)pThisServer->pCustomData;
        if (pCustom != NULL)
        {
            if( pCustom->pAdminNetwork != NULL )
            {
                delete pCustom->pAdminNetwork;
                pCustom->pAdminNetwork = NULL;
            }
            delete pCustom;
            pCustom = NULL;
        }
    }
    return SW_OK;
}

SWRESULT JoinServer(SWPluginServerStruct* pThisServer)
{   
    if (pThisServer)
    {
        CString strJoinApp;
        CString strJoinParams;
        CString strDir;
        CString strExe;
        CString strExt;
        CString strDrive;

        pThisServer->pSettings->BeginRead();
        strJoinApp = pThisServer->pSettings->GetJoinApp().c_str();
        strJoinParams = pThisServer->pSettings->GetJoinParamsFreeForm().c_str();
        pThisServer->pSettings->EndRead();
        
        _tsplitpath((LPCTSTR)strJoinApp, 
                    strDrive.GetBufferSetLength(_MAX_DRIVE), 
                    strDir.GetBufferSetLength(_MAX_DIR), 
                    strExe.GetBufferSetLength(_MAX_FNAME),
                    strExt.GetBufferSetLength(_MAX_EXT ));
        strDrive.ReleaseBuffer();
        strDir.ReleaseBuffer();
        strExe.ReleaseBuffer();
        strExt.ReleaseBuffer();

        strExe += strExt.GetBuffer(strExt.GetLength());
        strDir = strDrive + strDir;

        if (ShellExecute( NULL, _T("open"), strExe, 
						  strJoinParams, strDir, SW_SHOWNORMAL )
		    <= (HINSTANCE)ERROR_FILE_NOT_FOUND)
		{	
            MessageBox(NULL, _T("Unable to open specified application."), _T("Error"), MB_OK);
		}
    }
    return SW_OK;
}

SWRESULT LoadPlugin()
{
    g_pSettingsDlg = new CSettingsDlg;
                
    if (g_pSettingsDlg == NULL)
        return SW_MEMORYERR;

    g_pSettingsDlg->Create( NULL );

    _ASSERT( ::IsWindow(g_pSettingsDlg->GetPaneHWND()) );

    return SW_OK;
}

SWRESULT UnloadPlugin()
{
    ATLTRACE( L"UnloadPlugin\n" );
    
    if( NULL != g_pSettingsDlg )
    {
        if( ::IsWindow(g_pSettingsDlg->GetPaneHWND()) )
        {
            ::DestroyWindow( g_pSettingsDlg->GetPaneHWND() );
        }
            
        delete g_pSettingsDlg;
        g_pSettingsDlg = NULL;
    }

    return SW_OK;
}

SWRESULT CreateMenu(SWPluginServerStruct* /*pThisServer*/, ISWPluginMenu* /*pPluginMenu*/)
{
    ATLTRACE(_T("CreateMenu\n"));

    return SW_OK;
}