/*
    AL_Timer.h
    Copyright (c) 2000-2004, Deep Fried Software. All rights reserved.
 */

#if !defined(AFX_AL_TIMER_H__E1706DA0_A6FF_11D4_B433_DCAA1CE58E09__INCLUDED_)
#define AFX_AL_TIMER_H__E1706DA0_A6FF_11D4_B433_DCAA1CE58E09__INCLUDED_

#ifdef SWSERVICES_EXPORTS
#define SWSERVICES_API __declspec(dllexport)
#else
#define SWSERVICES_API __declspec(dllimport)
#endif

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "stdafx.h"
#include "SWMacros.h"
#include <time.h>
#include "SWHandle.h"
#include <process.h>

/**
 *  Inherit from this class to allow you to pass data classes through AL_Timer.
 *  Make sure you use the Retain/Release calls appropriately to do memory cleanup.
 *  After the class is passed in to AL_Timer you should release it.  You should
 *  retain it immediately whenever you recieve it from a message or risk having it
 *  destroy itself when you need it.
 *  <br><br>See Also: 
 *  \ref AL_Timer
 */
class SWSERVICES_API AL_Timer_MessageBase
{
	public:
		SWRESULT Retain();
		SWRESULT Release();
	protected:
		AL_Timer_MessageBase();
		virtual ~AL_Timer_MessageBase();
	private:
		unsigned int m_iRefCounter;
		HANDLE m_hRefMutex;
};

/**
 *  Timer
 *  <br><br>See Also: 
 *  \ref AL_Timer_MessageBase
 */
class SWSERVICES_API AL_Timer  
{
	struct TimeStruct
	{
		int                    iTime;           // From Init() (Time in ms)
		HWND                   hwndNotify;      // From Init()
		long                   NotifyMsg;       // From Init()
		AL_Timer_MessageBase*  Message;	        // From Init()
		HANDLE                 hAlert;          // Internal
		int                    iAlertResponse;  // Internal
		HANDLE                 hDataSafeMutex;  // Internal
		HANDLE                 hAlertRcvd;      // Internal
	};

	public:
		AL_Timer();
		virtual  ~AL_Timer();
		SWRESULT Init(int                   MillisecondTimeout,
					  void*                 Notifier,  //In windows this is an HWND
					  unsigned long         NotifyMsg,
					  AL_Timer_MessageBase* Message = NULL );
		SWRESULT SetNewTime(int newTime);
		SWRESULT Start();
		SWRESULT Stop();
		SWRESULT Pause();
		SWRESULT Resume();
	private:
		static void TimerProc(void*);
		TimeStruct* m_TimeData;
		HANDLE      m_TimerProcThread;
		bool        m_fIsInitialized;
	public:
};

#endif // !defined(AFX_AL_TIMER_H__E1706DA0_A6FF_11D4_B433_DCAA1CE58E09__INCLUDED_)
