/*
    swplugindllex.h
    Copyright (c) 2000-2004, Deep Fried Software. All rights reserved.
 */

#ifndef _SWPLUGINDLL_H
#define _SWPLUGINDLL_H

#ifndef DOXYGEN_SHOULD_SKIP_THIS

extern HINSTANCE g_hDll;
extern void OnProcessAttached(HINSTANCE g_hDll);
extern void OnProcessDetached();
extern HWND GetParentHWND();

#endif //DOXYGEN_SHOULD_SKIP_THIS

struct SWPluginServerStruct;
class ISWPluginMenu;

/**
    @file swplugindllex.h
    @brief The file defines the C functions that must be implemented by each plugin extender. They form
    the interface that allows Server Watch to communicate with your plugin.  They are very similar to
    that found in the plugin definitions, with some changes.

    @see SWPluginServerStruct
*/

/**
 * @name Loading and Unloading the Plugin
 * The plugin is given the oportunity to initialize any information global to the
 * plugin when it is loaded as well as clean itself up before it closes through
 * the implementation of these functions.
 */
//@{
    /**
    * @copydoc swplugindll::LoadPlugin()
    */
    SWRESULT LoadPlugin();
    /**
    * @copydoc swplugindll::UnloadPlugin()
    */
    SWRESULT UnloadPlugin();
//@}

/**
 * @name Introspective Functions
 * Server Watch needs to know what functionality is and is not supported by each
 * plugin, because not every plugin will support all of Server Watch's built in
 * functionality. In order for Server Watch to get this information, the plugin
 * must implement a set of introspection functions that allow Server Watch to
 * ask the plugin about itself, and what functionality it supports.
 */
//@{
    
    /**
     * The elements of this enum represent the possible
     * things that a plugins could be queried about support for
     */
    enum FunctionalSwitch
    {
        PING = 0, ///< Whether or not ping should be called on the plugin extender
        MENUS     ///< Whether or not the plugin extender adds menu items
    };

    /**
     * @copydoc swplugindll::DoYouSupport()
     */
    bool DoYouSupport(const FunctionalSwitch eType);

    /**
     * Called by Server Watch in order to ascertain what plugins should be
     * extended. Server Watch will call this function for each standard plugin
     * that has been loaded with the SWHandle identifier of that plugin.
     * @note You can get a plugins handle in "Settings" section of Server Watch
     * on the "About Plugins" pane. The text of the handle can be copied from that
     * pane to simplify this process.
     * @param pluginHandle The handle of the plugin in question.
     * @retval true The specified plugin is extended by this pluginex
     * @retval false ReturnThe specified plugin is not extended by this pluginex
     */
    bool DoYouExtend(const SWHandle* pluginHandle);
//@}

/** @name Creating and Destroying a Server
 *  @brief Functions that will be called when servers are created or destroyed
 */
//@{
    /**
     * @copydoc swplugindll::InitServer()
     */
    SWRESULT InitServer(SWPluginServerStruct* pThisServer);
    /**
     * @copydoc swplugindll::CleanupServer()
     */
    SWRESULT CleanupServer(SWPluginServerStruct* pThisServer);
//@}

/** @name Server Actions
 *  @brief These are a set of functions triggered directly by user action.
 */
//@{
    /**
     * @copydoc swplugindll::Ping()
     */
    SWRESULT Ping(SWPluginServerStruct* pThisServer);
//@}

/** @name Menu Actions
 *  Allows a plugin to display special context menus selections
 *  in key areas of Server Watch.
 */
//@{
    /**
     *  @copydoc swplugindll::CreateMenu()
     */
    SWRESULT CreateMenu(SWPluginServerStruct* pThisServer, 
                        ISWPluginMenu* pPluginMenu);
//@}
#endif // _SWPLUGINDLL_H